# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-02-25 19:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_rest_framework_apikeys', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='apikey',
            name='key',
            field=models.CharField(blank=True, max_length=40, null=True, unique=True),
        ),
    ]



from django.contrib.auth.models import User
from rest_framework import authentication, exceptions


class APIKeyAuthentication(authentication.BaseAuthentication):
    def authenticate(self, request):
        api_key = request.META.get("HTTP_X_API_KEY", None)

        if not api_key:
            return None

        try:
            user = User.objects.get(apikey__key=api_key)
        except User.DoesNotExist:
            raise exceptions.AuthenticationFailed('No such User')

        return (user, None)


    def authenticate_header(self, request):
        return "API_KEY"


from rest_framework import serializers
from .models import APIKey


class APIKeySerializer(serializers.ModelSerializer):
    class Meta:
        model = APIKey
        fields = "__all__"
        read_only_fields = ("user", "key")



from rest_framework import generics, mixins

from .models import APIKey
from .serializers import APIKeySerializer


class APIKeyList(mixins.ListModelMixin, generics.GenericAPIView):
    serializer_class = APIKeySerializer

    def get_queryset(self):
        user = self.request.user
        return APIKey.objects.filter(user=user)

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class APIKeyCreate(mixins.CreateModelMixin, generics.GenericAPIView):
    serializer_class = APIKeySerializer
    queryset = APIKey.objects.all()

    def perform_create(self, serializer):
        user = self.request.user
        serializer.save(user=user)

    def post(self, request, *args, **kwargs):
        return self.create(request, *args, **kwargs)


class APIKeyDelete(mixins.DestroyModelMixin, generics.GenericAPIView):
    serializer_class = APIKeySerializer
    queryset = APIKey.objects.all()
    lookup_url_kwarg = "id"

    def delete(self, request, *args, **kwargs):
        return self.destroy(request, *args, **kwargs)


from django.conf.urls import url

from .views import APIKeyList, APIKeyCreate, APIKeyDelete

urlpatterns = [
    url(r'^list/$', APIKeyList.as_view()),
    url(r'^create/$', APIKeyCreate.as_view()),
    url(r'^delete/(?P<id>[0-9a-f-]+)/$', APIKeyDelete.as_view()),

]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.contrib.auth.models import User

import uuid, binascii, os


class APIKey(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    user = models.ForeignKey(User)
    key = models.CharField(max_length=40, unique=True, blank=True, null=True)
    created = models.DateTimeField(auto_now_add=True)
    edited = models.DateTimeField(auto_now=True)

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = binascii.hexlify(os.urandom(20)).decode()
        return super(APIKey, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.key

